/*******************************************************************************
*    Comm2J 1.0.                                                               *
*    Copyright (C) 2004 Luca Bertoncello                                       *
*    Hartigstrae, 12 - 01127 Dresden Deutschland                              *
*    E-Mail: lucabert@lucabert.com, lucabert@lucabert.de                       *
*    http://www.lucabert.com/  http://www.lucabert.de/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

package comm2j;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;

/**
 * Message is the basic class used to send and receive message
 * to/from remote server.<br>
 * It will be derived to manage different type of message.<br>
 * <h3>Do <u>NOT</u> use this class directly!</h3>
 * @see Comm2J
 * @author Luca Bertoncello &lt;lucabert@lucabert.de&gt;
 * @version 1.0
*/

public abstract class Message implements MessageI
{
  protected boolean messageHasBeenSet = false;
  protected Object message = null;
  protected String className = null;

  /**
   * Constructor.
   */
  public Message()
  {
    this.className                  = "Message.class";
  }

  /**
   * Sets the message
   * @param message Message to be used
   */
  public void setMessage(Object message)
  {
    this.messageHasBeenSet          = true;
    this.message                    = message;
  }

  /**
   * Returns the current message
   * @return The current message
   */
  public Object getMessage()
  {
    return this.message;
  }

  /**
   * Reads a message from give InputStream.
   * @param is The InputStream used to read data
   * @return Number of read bytes
   * @throws java.io.IOException if an error occours
   */
  public int readMessage(InputStream is) throws IOException
  {
    DataInputStream dis;
    byte[] buf;

    this.messageHasBeenSet          = false;
    dis                             = new DataInputStream(is);
    int len                         = Read2J.readLength(dis);
    buf                             = new byte[len];
    for(int l = 0; l < len; l++)
      buf[l]                        = (byte) dis.read();

    setInternalObject(new String(buf));
    this.messageHasBeenSet          = true;

    return (len + Read2J.PACKLENGTH);
  }

  /**
   * Send current message throw given OutputStream
   * @param os The OutputStream used to read data
   * @return Number of wrote bytes
   * @throws java.io.IOException if an error occours
   */
  public int writeMessage(OutputStream os) throws IOException
  {
    DataOutputStream dos;
    String msg;

    dos                             = new DataOutputStream(os);
    msg                             = this.message.toString();
    Read2J.writeLength(dos, msg.length());
    for(int l = 0; l < msg.length(); l++)
      dos.writeByte(msg.charAt(l));

    return (msg.length() + Read2J.PACKLENGTH);
  }

  /**
   * Used to know if a message was set (by setMessage or writeMessage)
   * @return true is message was set
   */
  public boolean isMessageSet()
  {
    return this.messageHasBeenSet;
  }

  /**
   * Send an header identifying the message type
   * throw the give OutputStream
   * @param os The OutputStream used to read data
   * @return Number of wrote bytes
   * @throws java.io.IOException if an error occours
   */
  public int writeHeader(OutputStream os) throws IOException
  {
    DataOutputStream dos;

    dos                             = new DataOutputStream(os);
    Read2J.writeLength(dos, this.className.length());
    for(int l = 0; l < this.className.length(); l++)
      dos.writeByte(this.className.charAt(l));

    return (this.className.length() + Read2J.PACKLENGTH);
  }

  /**
   * Gets the header identifying the message type
   * from the give InputStream
   * @param is The InputStream used to read data
   * @return The message type
   * @throws java.io.IOException if an error occours
   */
  public String readHeader(InputStream is) throws IOException
  {
    DataInputStream dis;
    byte[] buf;

    dis                             = new DataInputStream(is);
    int len                         = Read2J.readLength(dis);
    buf                             = new byte[len];
    for(int l = 0; l < len; l++)
      buf[l]                        = (byte) dis.read();

    return new String(buf);
  }

  /**
   * Returns a string with the content of the message
   * @return A String identifying the message
   */
  public String toString()
  {
    return this.message.toString();
  }

  /**
   * Returns the name of the message class
   * @return The class name
   */
  public String getName()
  {
    return this.className;
  }

  /**
   * Sets the internal object.
   * This function is used by readMessage.<br>
   * Usually you don't need to use it directly.
   * @param obj Internal object to be set
   */
  protected abstract void setInternalObject(Object obj);
}
